/*
   DScope Application

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef MainWindow_H
#define MainWindow_H
//
#include <QMainWindow>
#include <QKeyEvent>
#include <QImage>
#include <QSplitter>
#include <QtNetwork> 
#include <QTcpSocket>
#include <vector>
#include "ui_mainwindow.h"
#include "DScopesQT.h"
#include "DScopesQTWidget.h"
#include "Scope.h"

#include <QExtSerialPort/qextserialport.h>

#include "FrameParser/FrameParser3.h"

//-------------------------------------------------------------------------------------------------
// Data structures
//-------------------------------------------------------------------------------------------------
typedef struct {
	string title;
	vector<unsigned> traces;
	vector<unsigned> colors;
	int xscale;
	vector<int> yscale;
	bool yauto;
} ScopeDefinition;

typedef struct {
	//unsigned numtraces;
	vector <ScopeDefinition> scopedefinition;
} ScopesDefinition;

//-------------------------------------------------------------------------------------------------
// MainWindow
//-------------------------------------------------------------------------------------------------
class MainWindow : public QMainWindow, public Ui::MainWindow
{
Q_OBJECT
public:
        MainWindow( QWidget * parent = 0, Qt::WFlags f = 0 );
protected:
	virtual void timerEvent(QTimerEvent *event);
private:
	DScopesQTWidget *dscopes;
	int Connected;	// 0: unconnected. 1: tcp. 2: serial
	QextSerialPort *port;
	QTcpSocket *tcpSocket;
	int timer;
	bool binary;			
   FrameParser3 fp;
   int totreceiveddata;

	QLabel *datarateLabel;
	QLabel *displayrateLabel;
	
	double pt;
	double lastplot,refresh;
	
	double time_received_last,time_received_delta;
	std::vector<double> time_received_delta_vector;
	
	double time_displayed_last,time_displayed_delta;
	
	void Plot();
	void receivedData(vector<int> &linedata);
	
	
	
	// Trace storage
	vector<vector<int> > alldata;
   ScopesDefinition sd;
	vector<unsigned> handles;
	//QMutex mutex;
	
	void BinaryChunckDecodeRead(QByteArray &ba);
	void TextChunckDecodeRead(QTextStream &in);
	
	void loadConfiguration(QString fileName);
   void SaveData();
   void DeleteAllScopes();
   void AddScopes(unsigned n);
	
private slots:
   void on_pushButton_ClearData_clicked();
   void on_pushButton_SaveData_clicked();
   void connectClicked();
	void applyClicked();
	void applyBinary();
	void testClicked();
	void testDashel();
	void tcpRead();
	void serialRead();
//	void stdioRead();
	void displayError(QAbstractSocket::SocketError);
	void SetRefreshRate(int i);
	void SetAfterGlow(int state);
	void dsrChanged(bool);
	void aboutToClose();
	void saveConfiguration();
	void loadConfigurationAction();
	void actionExit();
	void actionAbout();
	void actionListPorts();
	void actionHowto();
};
#endif




